#!/usr/bin/perl
# Test rest

use warnings;
use strict;
use FindBin qw($Bin);
use lib "$Bin/../lib", "$Bin/lib";

use lib "/usr/local/nagios/perl/lib", "/usr/local/nagios/lib", "/usr/local/nagios/etc", "/usr/local/nagios/t/lib";

use Test::More qw(no_plan);
use Test::Deep;
use Test::LongString;

use Opsview::Test;
use Opsview::TestUtils;
use Test::WWW::Mechanize;

use Data::Dump qw(dump);

my $res_data;
my $content;
my $expected;
# Use undef as poor ACLs combined with a saved cookie was allowing access
my $ua   = Test::WWW::Mechanize->new( stack_depth => 0, cookie_jar => undef );
$ua->add_header('content-type' => 'text/x-data-dumper' );
my $url_prefix="http://localhost:3000";

$ua->post( "$url_prefix/rest/login", Content => qq%{ username => "admin", password => "initial" }%, "Login to API2" );
is( $ua->status, 200 );
$ua->content_contains( "token", "Should return some ticket information" );
$content = $ua->content;
like( $content, qr%^\{'token' => '[0-9a-f]{40}'\}$%, "Got expected content" );
my $data = eval $content;
like( $data->{token}, qr/^[0-9a-f]{40}$/, "Got token value" );

$ua->add_header("X-Opsview-Username" => "admin");
$ua->add_header("X-Opsview-Token" => $data->{token});

$ua->get_ok( "$url_prefix/rest/acknowledge?rows=0" );
is( $ua->status, 200, "Got list of ack possibilities okay" ) || diag explain $ua->content;
$expected = {'summary' => { 'handled' => 10, 'unhandled' => 21, total => 31, service => { critical => 3, handled => 1, unknown => 18, unhandled => 20, total => 21}, host => { handled => 9, unhandled => 1, down => 1, up => 9, total => 10 } } };
$res_data = eval $ua->content;
is_deeply( $res_data, $expected, "Got expected summary" ) || diag explain $ua->content;

my $somehosts = Test::WWW::Mechanize->new( stack_depth => 0 );
rest_log_in( $somehosts, "somehosts", "somehosts", $url_prefix );
$somehosts->get_ok("/rest/acknowledge?rows=0");
$res_data = eval $somehosts->content;
$expected = {'summary' => { 'handled' => 1, 'unhandled' => 3, total => 4, service => { critical => 1, handled => 0, unknown => 2, unhandled => 3, total => 3}, host => { handled => 1, unhandled => 0, up => 1, total => 1 } } };
is_deeply( $res_data, $expected, "Summary data different from admin user, due to permissions") || diag explain $somehosts->content;

$ua->post("$url_prefix/rest/acknowledge", Content => '{comment=>""}' );
is( $ua->status, 403, "Got 403 error code for missing comment");
$res_data = eval $ua->content;
is( $res_data->{message}, "Error validating acknowledge parameters", "Got right error message" );
is( $res_data->{detail}, 'Comment is empty', "Got right detail" ) || diag($ua->content);

$ua->post("$url_prefix/rest/acknowledge", Content => '{comment=>"stuff"}' );
is( $ua->status, 403, "Got 403 error code for no objects chosen");
$res_data = eval $ua->content;
is( $res_data->{message}, "No objects chosen for acknowledgements", "Got right error message" ) || diag($ua->content);

# Test POSTs for specific objects
# 206=valid service id
$ua->post("$url_prefix/rest/acknowledge?hst.hostname=missinghost&hst.hostid=666&svc.hs=badhost::badservice&svc.serviceid=696969&svc.serviceid=206", Content => '{comment=>"autotest"}');
#$ua->post("$url_prefix/rest/acknowledge?hg.hostgroupid=99&hg.hostgroupname=missinghg&hst.hostname=missinghost&hst.hostid=666&svc.hs=badhost::badservice&svc.serviceid=696969&svc.serviceid=206&comment=autotest");
is( $ua->status, 403, "Bad objects" );
$res_data = eval $ua->content;
$expected = {
    message => "Error when searching for objects",
    detail => "Cannot find host with name missinghost; Cannot find host with id 666; Cannot find service with hs badhost::badservice; Cannot find service with serviceid 696969",
};
is_deeply( $res_data, $expected, "Get an error due to hostgroupid which doesn't exist" ) || diag explain $res_data;

# Check using just hostid and serviceid is sufficient
$ua->post("$url_prefix/rest/acknowledge?hst.hostid=6969&svc.serviceid=7777", Content => '{comment=>"autotest", starttime => "2129999900", endtime => "2129999999"}');
is( $ua->status, 403, "Bad objects" ) || diag explain $somehosts->content;
$res_data = eval $ua->content;
$expected = {
    message => "Error when searching for objects",
    detail => "Cannot find host with id 6969; Cannot find service with serviceid 7777",
};
is_deeply( $res_data, $expected, "Get an error due to objects which don't exist" ) || diag explain $res_data;


my $cmdfile="/usr/local/nagios/var/rw/nagios.cmd";
open F, ">", $cmdfile;
close F;
$ua->post("$url_prefix/rest/acknowledge?hst.hostname=opsview", Content => '{comment=>"autotest comment with bad < chars > within ;", starttime=>"2129999900", endtime=>"2129999999"}' );
is( $ua->status, 200, "Got ok for host downtime" );
$res_data = eval $ua->content;
$expected = {
    summary => {
        num_hosts => 1,
        num_services => 0,
    },
    list => {
        hosts => [ { hostname => "opsview", id => 116 } ],
        services => [],
    },
};
is_deeply( $res_data, $expected, "Got expected response" ) || diag($ua->content);

open F, $cmdfile;
my @data = map { s/^\[\d+\] //; chomp; $_ } <F>;
close F;
$expected = [
'ACKNOWLEDGE_HOST_PROBLEM;opsview;0;0;0;admin;autotest comment with bad  chars  within ',
];
is_deeply( \@data, $expected, "Got commands as expected" ) || diag explain \@data;



# Test readonly cannot post acks
my $readonly = Test::WWW::Mechanize->new( stack_depth => 0 );
rest_log_in( $readonly, "readonly", "readonly", $url_prefix );
$readonly->post("$url_prefix/rest/acknowledge?hst.hostname=opslave?comment=noaccess");
is( $readonly->status, 403, "No access" );
$res_data = eval $readonly->content;
is_deeply( $res_data, { message => "Access denied" }, "Expected error") || diag explain $res_data;





open F, ">", $cmdfile;
close F;
$ua->post("$url_prefix/rest/acknowledge?hst.hostname=cisco2&hst.hostname=cloned2&hst.hostid=3", Content => '{comment=>"host ack", starttime=>"2129999900", endtime=>"2129999999"}' );
is( $ua->status, 200, "Got ok for host ack" );
$res_data = eval $ua->content;
$expected = {
    summary => {
        num_hosts => 2,
        num_services => 0,
    },
    list => {
        hosts => [
            { id => 118, hostname => "cisco2" },
            { id => 3, hostname => "cloned2" },
        ],
        services => [],
    },
};
is_deeply( $res_data, $expected, "Got expected response" ) || diag($ua->content);

open F, $cmdfile;
@data = map { s/^\[\d+\] //; chomp; $_ } <F>;
close F;
$expected = [
'ACKNOWLEDGE_HOST_PROBLEM;cisco2;0;0;0;admin;host ack',
'ACKNOWLEDGE_HOST_PROBLEM;cloned2;0;0;0;admin;host ack',
];
is_deeply( \@data, $expected, "Got commands as expected" ) || diag explain \@data;


# Test somehosts has restricted ack access
# Also tests duration attribute
open F, ">", $cmdfile;
close F;
$somehosts->post("$url_prefix/rest/acknowledge?hst.hostname=cisco2&hst.hostname=cloned2&hst.hostname=monitored_by_slave&hst.hostid=3", Content => '{comment=>"host acknowledge", starttime=>"2129999900", endtime=>"+5h"}' );
is($somehosts->status, 403, "Error with restricted objects" );
$res_data = eval $somehosts->content;
$expected = {
    message => "Error when searching for objects",
    detail => "Cannot find host with name cisco2; Cannot find host with name cloned2; Cannot find host with id 3",
};
is_deeply( $res_data, $expected, "Get an error due to hostgroupid which doesn't exist" ) || diag explain $res_data;

$somehosts->post("$url_prefix/rest/acknowledge?hst.hostname=monitored_by_slave&svc.servicegroupname=unix", Content => '{comment=>"host acknowledge", starttime=>"2129999900", endtime=>"+5h"}' );
is( $somehosts->status, 200, "Got ok for host acknowledge" );
$res_data = eval $somehosts->content;
$expected = {
    summary => {
        num_hosts => 1,
        num_services => 1,
    },
    list => {
        hosts => [
            { id => 136, hostname => "monitored_by_slave" },
        ],
        services => [
            { id => 193, hostname => "monitored_by_slave", servicename => "/" },
        ],
    },
};
is_deeply( $res_data, $expected, "Got expected response" ) || diag($somehosts->content);
open F, $cmdfile;
@data = map { s/^\[\d+\] //; chomp; $_ } <F>;
close F;
$expected = [
'ACKNOWLEDGE_HOST_PROBLEM;monitored_by_slave;0;0;0;somehosts;host acknowledge',
'ACKNOWLEDGE_SVC_PROBLEM;monitored_by_slave;/;0;0;0;somehosts;host acknowledge',
];
is_deeply( \@data, $expected, "Got commands as expected" ) || diag explain \@data;



# This also tests that data has priority over URL parameters
open F, ">", $cmdfile;
close F;
$ua->post("$url_prefix/rest/acknowledge?svc.hostname=cisco2&svc.hostname=%4&svc.servicename=Test exceptions&svc.servicename=/&svc.hostname=opslave&starttime=2129999900&endtime=2129999988", Content => "{comment=>'svc acknowledge\nwith linefeed', endtime=>'2129999999'}" );
is( $ua->status, 200, "Got ok for svc acknowledge" );
$res_data = eval $ua->content;
$expected = {
    summary => {
        num_hosts => 0,
        num_services => 3,
    },
    list => {
        hosts => [],
        services => [
            { id => 156, hostname => "cisco2", servicename => "Test exceptions" },
            { id => 162, hostname => "cisco4", servicename => "Test exceptions" },
            { id => 197, hostname => "opslave", servicename => "/" },
         ],
     },
};
is_deeply( $res_data, $expected, "Got expected response" ) || diag($ua->content);

open F, $cmdfile;
@data = map { s/^\[\d+\] //; chomp; $_ } <F>;
close F;
$expected = [
'ACKNOWLEDGE_SVC_PROBLEM;cisco2;Test exceptions;0;0;0;admin;svc acknowledgewith linefeed',
'ACKNOWLEDGE_SVC_PROBLEM;cisco4;Test exceptions;0;0;0;admin;svc acknowledgewith linefeed',
'ACKNOWLEDGE_SVC_PROBLEM;opslave;/;0;0;0;admin;svc acknowledgewith linefeed',
];
is_deeply( \@data, $expected, "Got commands as expected" ) || diag explain \@data;


# Test keyword searching
open F, ">", $cmdfile;
close F;
$ua->post("$url_prefix/rest/acknowledge?svc.keywordname=cisco_gp2&svc.servicename=Coldstart&starttime=2129999900", Content => "{comment=>'svc acknowledge\nwith linefeed', endtime=>'2129999999'}" );
is( $ua->status, 200, "Got ok for svc acknowledge" );
$res_data = eval $ua->content;
$expected = {
    summary => {
        num_hosts => 0,
        num_services => 3,
    },
    list => {
        hosts => [],
        services => [
            { id => 155, hostname => "cisco2", servicename => "Coldstart" },
            { id => 158, hostname => "cisco3", servicename => "Coldstart" },
            { id => 161, hostname => "cisco4", servicename => "Coldstart" },
         ],
     },
};
is_deeply( $res_data, $expected, "Got expected response" ) || diag($ua->content);

open F, $cmdfile;
@data = map { s/^\[\d+\] //; chomp; $_ } <F>;
close F;
$expected = [
'ACKNOWLEDGE_SVC_PROBLEM;cisco2;Coldstart;0;0;0;admin;svc acknowledgewith linefeed',
'ACKNOWLEDGE_SVC_PROBLEM;cisco3;Coldstart;0;0;0;admin;svc acknowledgewith linefeed',
'ACKNOWLEDGE_SVC_PROBLEM;cisco4;Coldstart;0;0;0;admin;svc acknowledgewith linefeed',
];
is_deeply( \@data, $expected, "Got commands as expected" ) || diag explain \@data;


# Test keyword searching by id
open F, ">", $cmdfile;
close F;
$ua->post("$url_prefix/rest/acknowledge?svc.keywordid=4&svc.servicename=Coldstart&starttime=2129999900", Content => "{comment=>'svc acknowledge\nwith linefeed', endtime=>'2129999999'}" );
is( $ua->status, 200, "Got ok for svc acknowledge" );
$res_data = eval $ua->content;
$expected = {
    summary => {
        num_hosts => 0,
        num_services => 3,
    },
    list => {
        hosts => [],
        services => [
            { id => 155, hostname => "cisco2", servicename => "Coldstart" },
            { id => 158, hostname => "cisco3", servicename => "Coldstart" },
            { id => 161, hostname => "cisco4", servicename => "Coldstart" },
         ],
     },
};
is_deeply( $res_data, $expected, "Got expected response" ) || diag($ua->content);

open F, $cmdfile;
@data = map { s/^\[\d+\] //; chomp; $_ } <F>;
close F;
$expected = [
'ACKNOWLEDGE_SVC_PROBLEM;cisco2;Coldstart;0;0;0;admin;svc acknowledgewith linefeed',
'ACKNOWLEDGE_SVC_PROBLEM;cisco3;Coldstart;0;0;0;admin;svc acknowledgewith linefeed',
'ACKNOWLEDGE_SVC_PROBLEM;cisco4;Coldstart;0;0;0;admin;svc acknowledgewith linefeed',
];
is_deeply( \@data, $expected, "Got commands as expected" ) || diag explain \@data;


# Test host group searching - search by id and by hostgroup name at same time
open F, ">", $cmdfile;
close F;
$ua->post("$url_prefix/rest/acknowledge?svc.servicecheckid=94&svc.hostgroupname=Monitoring+Servers&svc.hostgroupid=2&starttime=2129999900&endtime=2129999988", Content => "{comment=>'svc acknowledge\nwith linefeed', endtime=>'2129999999'}" );
is( $ua->status, 200, "Got ok for svc acknowledge" );
$res_data = eval $ua->content;
$expected = {
    summary => {
        num_hosts => 0,
        num_services => 2,
    },
    list => {
        hosts => [],
        services => [
            { id => 219, hostname => "opsview", servicename => "Slave-node: opslave" },
            { id => 220, hostname => "opsview", servicename => "Slave-node: opslaveclusterA" },
         ],
     },
};
is_deeply( $res_data, $expected, "Got expected response" ) || diag($ua->content);

open F, $cmdfile;
@data = map { s/^\[\d+\] //; chomp; $_ } <F>;
close F;
$expected = [
'ACKNOWLEDGE_SVC_PROBLEM;opsview;Slave-node: opslave;0;0;0;admin;svc acknowledgewith linefeed',
'ACKNOWLEDGE_SVC_PROBLEM;opsview;Slave-node: opslaveclusterA;0;0;0;admin;svc acknowledgewith linefeed',
];
is_deeply( \@data, $expected, "Got commands as expected" ) || diag explain \@data;


# Test somehosts will only be allowed to set downtime for monitored_by_slave::VNC
open F, ">", $cmdfile;
close F;
$somehosts->post("$url_prefix/rest/acknowledge?svc.hostname=cisco2&svc.hostname=cisco4&svc.servicename=Test exceptions&svc.hs=monitored_by_slave::VNC&starttime=2129999900&endtime=2129999988", Content => "{comment=>'svc acknowledge\nwith linefeed', endtime=>'2129999999', notify => '1', sticky => '1'}" );
is( $somehosts->status, 200, "Got ok for svc acknowledge" );
$res_data = eval $somehosts->content;
$expected = {
    summary => {
        num_hosts => 0,
        num_services => 1,
    },
    list => {
        hosts => [],
        services => [
            { id => 5, hostname => "monitored_by_slave", servicename => "VNC" },
         ],
     },
};
is_deeply( $res_data, $expected, "Got expected response" ) || diag($somehosts->content);
open F, $cmdfile;
@data = map { s/^\[\d+\] //; chomp; $_ } <F>;
close F;
$expected = [
'ACKNOWLEDGE_SVC_PROBLEM;monitored_by_slave;VNC;1;1;0;somehosts;svc acknowledgewith linefeed',
];
is_deeply( \@data, $expected, "Got commands as expected" ) || diag explain \@data;





# Test searching using status parameters
open F, ">", $cmdfile;
close F;
$ua->post("$url_prefix/rest/acknowledge?hostname=opsview&servicecheck=Slave-node:%25", Content => "{comment=>'svc acknowledge\nwith linefeed', endtime=>'2129999999'}" );
is( $ua->status, 200, "Checking status parameters searching" );
$res_data = eval $ua->content;
$expected = {
    summary => {
        num_hosts => 0,
        num_services => 2,
    },
    list => {
        hosts => [],
        services => [
            { id => 219, hostname => "opsview", servicename => "Slave-node: opslave" },
            { id => 220, hostname => "opsview", servicename => "Slave-node: opslaveclusterA" },
         ],
     },
};
is_deeply( $res_data, $expected, "Got expected response" ) || diag($ua->content);

open F, $cmdfile;
@data = map { s/^\[\d+\] //; chomp; $_ } <F>;
close F;
$expected = [
'ACKNOWLEDGE_SVC_PROBLEM;opsview;Slave-node: opslave;0;0;0;admin;svc acknowledgewith linefeed',
'ACKNOWLEDGE_SVC_PROBLEM;opsview;Slave-node: opslaveclusterA;0;0;0;admin;svc acknowledgewith linefeed',
];
is_deeply( \@data, $expected, "Got commands as expected" ) || diag explain \@data;

