#!perl

use warnings;
use strict;

use FindBin qw($Bin);
use lib "$Bin/lib";
use lib "/usr/local/nagios/lib";

use Test::WWW::Mechanize;
use HTTP::Cookies;
use Sys::Hostname;

use Opsview::Web::Test;

use Opsview::Schema;
use Opsview::Config;

use Test::More qw(no_plan);
use Test::Deep;

my $rs = Opsview::Schema->my_connect->resultset("Contacts");

my $ua = Test::WWW::Mechanize->new( autocheck => 0 );
my $cj = HTTP::Cookies->new();
$ua->cookie_jar($cj);

my $hostname = Sys::Hostname::hostname;
$hostname = "localhost:3000";

# No cookies here - should get login page
my $res;


$res = $ua->get("http://$hostname/login?login_username=asdf&login_password=asdf&back=&app=%3Cscript%3Ealert%281%29%3C/script%3E&login=Log+In");
ok( $res->is_success, "Got login page");
$ua->content_lacks('<script>alert(1)</script>', "Should not have the hack passed through");

$res = $ua->get("http://$hostname/");
ok( $res->is_success, "Got successful response" );
is( $res->code, 200, "Status 200" );
like( $res->content, "/login_username/s", "Returned login_username, so is a login page" );
my $content = $res->content;
$content =~ s%<noscript>.*?</noscript>%%s;    # Remove "JavaScript is required message"
unlike( $content, "/<h3>/s", "Check that no login error is displayed" );

cmp_deeply( $cj->{COOKIES}, {}, "Cookie jar is empty" );

$ua->field( 'login_username', 'admin' );
$ua->field( 'login_password', 'initial' );
$ua->field( "noscript" => 0 );
$ua->current_form->action;
$ua->submit;

my $hh_title = qr/Host Group Summary &gt; Opsview/;

ok( $ua->res->is_success, "Successful reply" );
like( $ua->content, $hh_title, "Found heading for HH page" );

# Check cookies created
my $cookies_hash = $cj->{COOKIES}->{"localhost.local"}->{"/"};
ok( defined $cookies_hash->{auth_tkt},            "auth_tkt cookie assigned" );
ok( defined $cookies_hash->{opsview_web_session}, "opsview_web_session cookie assigned" );

my $saved_auth_tkt = $cookies_hash->{auth_tkt};
my $saved_session  = $cookies_hash->{opsview_web_session};

# Corrupt opsview_web_session cookie
# This should return a soft_error page
my $corrupt_opsview_web_session_cookie = [@$saved_session];
$corrupt_opsview_web_session_cookie->[1] = '1>\'><ScrRiPt%20%0d%0a>alert(482607104790)%3B</ScRiPt>';
$cookies_hash->{auth_tkt} = $saved_auth_tkt;
$cookies_hash->{opsview_web_session} = $corrupt_opsview_web_session_cookie;
$ua->get("http://$hostname/event");
is($ua->status, 400, "Got special page due to invalid session cookie" );
unlike( $ua->content, qr/<ScrRiPt/, "Check we haven't printed text to screen" );
unlike( $ua->content, qr/&ltScrRiPt/, "Ditto" );
like( $ua->content, qr/Invalid session ID/, "Check special error message" );

# Remove auth_tkt cookie
# If only session cookie, get login page
delete $cookies_hash->{auth_tkt};
$ua->get("http://$hostname/status/hostgroup");
like( $ua->content, "/login_username/s", "Got login page due to no auth_tkt" );

# Corrupt the auth_tkt. Check that error message includes invalid ticket
my $corrupt_auth_tkt = [@$saved_auth_tkt];
$corrupt_auth_tkt->[1] = 'somerubbishvaluewhichwillnevergetvalidatedcorrectly';
$cookies_hash->{auth_tkt} = $corrupt_auth_tkt;
$ua->get("http://$hostname/status/hostgroup");
like( $ua->content, "/login_username/", "Got login page" );
like( $ua->content, "/invalid authentication ticket/i", "States invalid authentication ticket" );
ok( !exists $cookies_hash->{auth_tkt}, "auth_tkt was deleted" );

# Hack the auth_tkt. Since we know the shared secret,
# we can create dummy tickets
use Apache::AuthTkt;
my $tkt = Apache::AuthTkt->new( secret => Opsview::Config->authtkt_shared_secret, );
my $hacked_auth_tkt = $corrupt_auth_tkt;
$cookies_hash->{auth_tkt} = $hacked_auth_tkt;

# Default ticket expiry is 3600
$hacked_auth_tkt->[1] = $tkt->ticket( uid => "admin", ip_addr => "127.0.0.1", ts => time() - 3800 );
$ua->get("http://$hostname/status/hostgroup");
like( $ua->content, $hh_title, "Got status pages - if this fails, check shared secret" );
isnt( $cookies_hash->{auth_tkt}->[1], $hacked_auth_tkt->[1], "And auth ticket has been regenerated" );

# Have an expired auth ticket
$hacked_auth_tkt->[1] = $tkt->ticket( uid => "admin", ip_addr => "127.0.0.1", ts => time() - 3800000 );
$cookies_hash->{auth_tkt} = $hacked_auth_tkt;
$ua->get("http://$hostname/status/hostgroup");
is( $ua->status, 200, "Got 200 status for login page" );
like( $ua->content, "/login_username/", "Got login page" );
like( $ua->content, "/Authentication ticket has expired/i", "Displayed error of expired ticket" );
ok( !exists $cookies_hash->{auth_tkt}, "auth ticket deleted" );

# Test that a json call will return a 401 instead of the usual 200 with login page
$hacked_auth_tkt->[1] = $tkt->ticket( uid => "admin", ip_addr => "127.0.0.1", ts => time() - 3800000 );
$cookies_hash->{auth_tkt} = $hacked_auth_tkt;
$ua->get("http://$hostname/event?output=json");
is( $ua->status, 401, "Got 401 status" );

# NOTE: This is no longer checked because of authtkt_default_username
# Have a ticket with a mismatch of uid
#$hacked_auth_tkt->[1] = $tkt->ticket( uid => "differentadmin", ip_addr => "127.0.0.1" );
#$cookies_hash->{auth_tkt} = $hacked_auth_tkt;
#$cookies_hash->{opsview_web_session} = $saved_session;
#$ua->get("http://$hostname/status/hostgroup");
#like( $ua->content, "/login_username/", "Got login page" );
#like( $ua->content, "/authentication ticket does not match with existing session user/i", "Got mismatch of usernames error" );

# Create ticket with wrong ip address
$cookies_hash->{auth_tkt} = $hacked_auth_tkt;
$hacked_auth_tkt->[1] = $tkt->ticket( uid => "admin", ip_addr => "127.10.10.10" );
$ua->get("http://$hostname/status/hostgroup");
like( $ua->content, "/login_username/", "Got login page because wrong IP entered in ticket" );
like( $ua->content, "/Invalid authentication ticket/i", "Displayed error of invalid authentication ticket" );

# If only auth_tkt, but no session, retrieve user from db
# If user exists, allow...
delete $cookies_hash->{opsview_web_session};
$cookies_hash->{auth_tkt} = $saved_auth_tkt;
$ua->get("http://$hostname/status/hostgroup");
like( $ua->content, $hh_title, "Found heading for HH page" );
is( $cookies_hash->{auth_tkt}->[1], $saved_auth_tkt->[1], "auth ticket has not been regenerated" );

# ...but if web session is valid and user does not exist, need to get a soft error
$rs->search( { name => "admin" } )->delete;
$ua->get("http://$hostname/status/hostgroup");
like( $ua->content, "/login_username/", "Got login page for user with valid session but not on system" );
like( $ua->content, "/Authentication ticket found, but user does not exist/", "Got appropriate error" );

# ...otherwise, block if just auth tkt
delete $cookies_hash->{opsview_web_session};
$cookies_hash->{auth_tkt} = $saved_auth_tkt;

# If user doesn't exist, show login page with error message
# TODO: If a user is deleted with a valid session, gets a soft error in UI
$ua->get("http://$hostname/status/hostgroup");
like( $ua->content, "/login_username/", "Got login page for user not on system" );
like( $ua->content, "/Authentication ticket found, but user does not exist/s", "States user does not exist" );

# Test with a newly generated ticket cookie. Add the cookie directly to the request
use HTTP::Request;
my $request = HTTP::Request->new( GET => "http://$hostname/status/hostgroup?parentid=1" );
$request->header( "Cookie" => $tkt->cookie( uid => "nonadmin", ip_addr => "127.0.0.1" ) );
$ua = LWP::UserAgent->new;
my $response = $ua->request($request);
like( $response->content, $hh_title, "Found HH page from self generated cookie - if fails, check shared secret matches" );

# Try same, but use ip_addr as 0.0.0.0. This should fail because Opsview will use the incoming ip address
$request = HTTP::Request->new( GET => "http://$hostname/status/hostgroup" );
$request->header( "Cookie" => $tkt->cookie( uid => "nonadmin", ip_addr => "0.0.0.0" ) );
$ua       = LWP::UserAgent->new;
$response = $ua->request($request);
like( $response->content, "/login_username/", "Found login page because authtkt came in with 0.0.0.0 ip address" );
like( $response->content, "/Invalid authentication ticket/i", "Displayed error of invalid authentication ticket" );

# This test is to prove that a user which is deleted in the backend will not be
# allowed access in again, even if they have an authtkt and a session cookie
$ua  = WWW::Mechanize->new;
$res = $ua->get("http://$hostname/");
ok( $res->is_success, "Got successful response" );
is( $res->code, 200, "Status 200" );
like( $res->content, "/login_username/s", "Returned login_username, so is a login page" );
$content = $res->content;
$content =~ s%<noscript>.*?</noscript>%%s;    # Remove "JavaScript is required message"
unlike( $content, "/<h3>/s", "Check that no login error is displayed" );

$ua->field( 'login_username', 'somehosts' );
$ua->field( 'login_password', 'somehosts' );
$ua->field( "noscript" => 0 );
$ua->current_form->action;
$ua->submit;

ok( $ua->res->is_success, "Successful reply" );

$ua->get("http://$hostname/");
like( $ua->content, $hh_title );

$rs->search( { name => "somehosts" } )->delete;
$ua->get("http://$hostname/");
like( $ua->content, "/login_username/s", "Login page" );
like( $ua->content, "/Authentication ticket found, but user does not exist/" );
